@@ -569,8 +569,8 @@ class ColorDetectionUseCase(BaseProcessor):
                 track_id = rec.get('detection_id')
             if cat and track_id is not None:
                 # Update the color_det_dict with the actual color
-                if color_conf > self.color_det_dict[track_id][1] and track_id in self.color_det_dict:
-                    self.color_det_dict[track_id] = [color,color_conf]
+                # if color_conf > self.color_det_dict[track_id][1] and track_id in self.color_det_dict:
+                #     self.color_det_dict[track_id] = [color,color_conf]
                 if color and track_id not in self.color_det_dict:
                     self.color_det_dict[track_id] = [color,color_conf]
                     key = f"{cat}:{color}" if color else cat
