@@ -343,7 +343,7 @@ class FireSmokeUseCase(BaseProcessor):
                     if count > threshold:  # Fixed logic: alert when EXCEEDING threshold
                         alerts.append({
                             "alert_type": getattr(config.alert_config, 'alert_type', ['Default']) if hasattr(config.alert_config, 'alert_type') else ['Default'],
-                            "alert_id": "alert_"+category+'_'+str(alert_id),
+                            "alert_id": "alert_"+category+'_'+alert_serial+'_'+str(alert_id),
                             "incident_category": self.CASE_TYPE,
                             "threshold_level": threshold,
                             "ascending": get_trend(self._ascending_alert_list, lookback=23, prior=14),
@@ -644,13 +644,19 @@ class FireSmokeUseCase(BaseProcessor):
 
         if len(self.id_hit_list)==1:
             last_ending_id, incident_id = self._get_alert_incident_ids("",stream_info)
-            print('last_ending_id',last_ending_id)
+            print('last_ending_id',last_ending_id, incident_id, self.return_id_counter)
             if len(self.id_timing_list)>0 and len(self.id_timing_list)>=5:
                     start_timestamp = self.id_timing_list[-1]
+            if incident_id==self.return_id_counter:
+                incident_id = incident_id-1 
+            if self.return_id_counter > incident_id:
+                incident_id = self.return_id_counter-incident_id
             if last_ending_id==5:
+                alert_serial = getattr(config.alert_config, 'alert_type', ['Default']) if hasattr(config.alert_config, 'alert_type') else ['Default']
+                alert_serial = alert_serial[0]
                 alerts=[{
                             "alert_type": getattr(config.alert_config, 'alert_type', ['Default']) if hasattr(config.alert_config, 'alert_type') else ['Default'],
-                            "alert_id": "alert_"+'Event_Ended'+'_'+str(1),
+                            "alert_id": "alert_"+'Event_Ended'+'_'+alert_serial+'_'+str(incident_id),
                             "incident_category": self.CASE_TYPE,
                             "threshold_level": 0,
                             "ascending": False,
@@ -659,7 +665,8 @@ class FireSmokeUseCase(BaseProcessor):
                                     }       
                         }]
                 tracking_stats.append(alerts)
-                tracking_stats.append(self.create_incident(incident_id=self.CASE_TYPE+'_'+str(incident_id), incident_type=self.CASE_TYPE,
+                tracking_stats[0]['alerts']=alerts
+                tracking_stats.append(self.create_incident(incident_id='incident_'+self.CASE_TYPE+'_'+str(incident_id), incident_type=self.CASE_TYPE,
                             severity_level='info', human_text='Event Over', camera_info=camera_info, alerts=alerts, alert_settings=alert_settings,
                             start_time=start_timestamp, end_time='Incident still active',
                             level_settings= {"low": 3, "medium": 5, "significant":15, "critical": 30}))
