@@ -5,9 +5,6 @@ from datetime import datetime, timezone
 import copy
 import tempfile
 import os
-import cv2
-import numpy as np
-import torch
 from ..core.base import BaseProcessor, ProcessingContext, ProcessingResult, ConfigProtocol
 from ..utils import (
     filter_by_confidence,
@@ -21,10 +18,19 @@ from ..utils import (
     BBoxSmoothingConfig,
     BBoxSmoothingTracker
 )
-from ..ocr.easyocr_extractor import EasyOCRExtractor
-from ..ocr.postprocessing import TextPostprocessor 
-from ..ocr.preprocessing import ImagePreprocessor
-from ..core.config import BaseConfig, AlertConfig, ZoneConfig
+try:
+    import cv2
+    import numpy as np
+    import torch
+    from ..ocr.easyocr_extractor import EasyOCRExtractor
+    from ..ocr.postprocessing import TextPostprocessor 
+    from ..ocr.preprocessing import ImagePreprocessor
+    from ..core.config import BaseConfig, AlertConfig, ZoneConfig
+    
+except:
+    print("test-------------------------------------------------")
+
+
 
 @dataclass
 class LicensePlateMonitorConfig(BaseConfig):
@@ -138,7 +144,7 @@ class LicensePlateMonitorUseCase(BaseProcessor):
             
             # Initialize OCR extractor if not already done
             if self.ocr_extractor is None:
-                self.ocr_extractor = EasyOCRExtractor(lang=config.language, gpu=torch.cuda.is_available())
+                self.ocr_extractor = EasyOCRExtractor(lang=config.language, gpu=torch.cuda.is_available(), verbose=False)
             
             input_format = match_results_structure(data)
             context.input_format = input_format
